@@ -7,6 +7,7 @@ from jal.constants import Setup, BookAccount, TransactionType, TransferSubtype,
     CorporateAction, PredefinedCategory, PredefinedPeer
 from jal.db.helpers import executeSQL, readSQL, readSQLrecord, db_triggers_disable, db_triggers_enable
 from jal.db.db import JalDB
+from jal.db.settings import JalSettings
 from jal.ui.ui_rebuild_window import Ui_ReBuildDialog
 
 
@@ -42,8 +43,11 @@ class RebuildDialog(QDialog, Ui_ReBuildDialog):
 # Subclasses dictionary to store last amount/value for [book, account, asset]
 # Differs from dictionary in a way that __getitem__() method uses DB-stored values for initialization
 class LedgerAmounts(dict):
-    def __init__(self, *args, **kwargs):
+    def __init__(self, total_field=None, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        if total_field is None:
+            raise ValueError("Unitialized field in LedgerAmounts")
+        self.total_field = total_field
 
     def __getitem__(self, key):
         # predefined indices in key tuple
@@ -54,9 +58,9 @@ class LedgerAmounts(dict):
         try:
             return super().__getitem__(key)
         except KeyError:
-            amount = readSQL("SELECT sum_amount FROM ledger_sums "
+            amount = readSQL(f"SELECT {self.total_field} FROM ledger "
                              "WHERE book_account = :book AND account_id = :account_id AND asset_id = :asset_id "
-                             "ORDER BY sid DESC LIMIT 1",
+                             "ORDER BY id DESC LIMIT 1",
                              [(":book", key[BOOK]), (":account_id", key[ACCOUNT]), (":asset_id", key[ASSET])])
             amount = float(amount) if amount is not None else 0.0
             super().__setitem__(key, amount)
@@ -71,11 +75,9 @@ class Ledger(QObject):
     def __init__(self):
         QObject.__init__(self)
         self.current = {}
-        self.current_seq = -1
-        self.amounts = LedgerAmounts()    # store last amount for [book, account, asset]
-        self.values = LedgerAmounts()     # together with corresponding value
-        self.sids = {}                    # and keep sid of operation that modified it
-        self.main_window =None
+        self.amounts = LedgerAmounts("amount_acc")    # store last amount for [book, account, asset]
+        self.values = LedgerAmounts("value_acc")      # together with corresponding value
+        self.main_window = None
         self.progress_bar = None
 
     def setProgressBar(self, main_window, progress_widget):
@@ -94,7 +96,8 @@ class Ledger(QObject):
     #    of money in current account currency. Otherwise Amount contains only money value.
     # Method uses Account, Asset,Peer, Category and Tag values from current transaction
     def appendTransaction(self, book, amount, value=None):
-        seq_id = self.current_seq
+        op_type = self.current['type']
+        op_id = self.current['id']
         timestamp = self.current['timestamp']
         if book == BookAccount.Assets:
             asset_id = self.current['asset']
@@ -115,34 +118,15 @@ class Ledger(QObject):
         if (abs(amount) + abs(value)) <= (4 * Setup.CALC_TOLERANCE):
             return  # we have zero amount - no reason to put it into ledger
 
-        _ = executeSQL("INSERT INTO ledger (timestamp, sid, book_account, asset_id, account_id, "
-                       "amount, value, peer_id, category_id, tag_id) "
-                       "VALUES(:timestamp, :sid, :book, :asset_id, :account_id, "
-                       ":amount, :value, :peer_id, :category_id, :tag_id)",
-                       [(":timestamp", timestamp), (":sid", seq_id), (":book", book), (":asset_id", asset_id),
-                        (":account_id", account_id), (":amount", amount), (":value", value),
+        _ = executeSQL("INSERT INTO ledger (timestamp, op_type, operation_id, book_account, asset_id, account_id, "
+                       "amount, value, amount_acc, value_acc, peer_id, category_id, tag_id) "
+                       "VALUES(:timestamp, :op_type, :operation_id, :book, :asset_id, :account_id, "
+                       ":amount, :value, :amount_acc, :value_acc, :peer_id, :category_id, :tag_id)",
+                       [(":timestamp", timestamp), (":op_type", op_type), (":operation_id", op_id),
+                        (":book", book), (":asset_id", asset_id), (":account_id", account_id), (":amount", amount),
+                        (":value", value), (":amount_acc", self.amounts[(book, account_id, asset_id)]),
+                        (":value_acc", self.values[(book, account_id, asset_id)]),
                         (":peer_id", peer_id), (":category_id", category_id), (":tag_id", tag_id)])
-        try:
-            old_sid = self.sids[(book, account_id, asset_id)]
-        except KeyError:
-            old_sid = -1
-        if seq_id == old_sid:
-            _ = executeSQL("UPDATE ledger_sums SET sum_amount = :new_amount, sum_value = :new_value"
-                           " WHERE sid = :sid AND book_account = :book"
-                           " AND asset_id = :asset_id AND account_id = :account_id",
-                           [(":new_amount", self.amounts[(book, account_id, asset_id)]),
-                            (":new_value", self.values[(book, account_id, asset_id)]), (":sid", seq_id),
-                            (":book", book), (":asset_id", asset_id), (":account_id", account_id)], commit=True)
-        else:
-            _ = executeSQL("INSERT INTO ledger_sums(sid, timestamp, book_account, "
-                           "asset_id, account_id, sum_amount, sum_value) "
-                           "VALUES(:sid, :timestamp, :book, :asset_id, "
-                           ":account_id, :new_amount, :new_value)",
-                           [(":sid", seq_id), (":timestamp", timestamp), (":book", book), (":asset_id", asset_id),
-                            (":account_id", account_id), (":new_amount", self.amounts[(book, account_id, asset_id)]),
-                            (":new_value", self.values[(book, account_id, asset_id)])],
-                           commit=True)
-            self.sids[(book, account_id, asset_id)] = seq_id
 
     # Returns Amount measured in current account currency or asset that 'book' has at current ledger frontier
     def getAmount(self, book, asset_id=None):
@@ -229,10 +213,10 @@ class Ledger(QObject):
     # Process buy or sell operation base on self.current['amount'] (>0 - buy, <0 - sell)
     def processTrade(self):
         if self.current['peer'] == '':
-            logging.error(self.tr("Can't process trade as bank isn't set for investment account"))
+            logging.error(self.tr("Can't process trade as bank isn't set for investment account: ") +
+                          JalDB().get_account_name(self.current['account']))
             return
 
-        seq_id = self.current_seq
         account_id = self.current['account']
         asset_id = self.current['asset']
         type = copysign(1, self.current['amount'])  # 1 is buy, -1 is sell
@@ -246,102 +230,27 @@ class Ledger(QObject):
         # Get asset amount accumulated before current operation
         asset_amount = self.getAmount(BookAccount.Assets, asset_id)
         if ((-type) * asset_amount) > 0:  # Process deal match if we have asset that is opposite to operation
-            last_type = readSQL("SELECT s.type FROM deals AS d "
-                                "LEFT JOIN sequence AS s ON d.close_sid=s.id "
-                                "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
-                                "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
-                                [(":account_id", account_id), (":asset_id", asset_id)])
-            if last_type is None or last_type == TransactionType.Trade:
-                # Get information about last deal with quantity of opposite sign
-                last_sid = readSQL("SELECT "
-                                   "CASE WHEN (:type)*qty<0 THEN open_sid ELSE close_sid END AS last_sid "
-                                   "FROM deals "
-                                   "WHERE account_id=:account_id AND asset_id=:asset_id "
-                                   "ORDER BY close_sid DESC, open_sid DESC LIMIT 1",
-                                   [(":type", type), (":account_id", account_id), (":asset_id", asset_id)])
-                last_sid = 0 if last_sid is None else last_sid
-                # Next get information about abs trade quantity that was in this last deal
-                # It may be a corporate action - its quantity calculation is a bit more complicated
-                last_qty = readSQL("SELECT coalesce(SUM(qty), 0) AS qty FROM ( "
-                                   "SELECT ABS(t.qty) AS qty "
-                                   "FROM sequence AS s "
-                                   "LEFT JOIN trades AS t ON t.id=s.operation_id AND s.type=3 "
-                                   "WHERE s.id=:last_sid "
-                                   "UNION ALL "
-                                   "SELECT "
-                                   "CASE "
-                                   "    WHEN ca.type = 2 AND ca.asset_id=:asset_id THEN ca.qty "
-                                   "    ELSE ca.qty_new "
-                                   "END AS qty "
-                                   "FROM sequence AS s "
-                                   "LEFT JOIN corp_actions AS ca ON ca.id=s.operation_id AND s.type=5 "
-                                   "WHERE s.id=:last_sid "
-                                   ")",
-                                   [(":asset_id", asset_id), (":last_sid", last_sid)])
-                # Collect quantity of all deals where this last opposite trade participated (positive value)
-                # If it was a corporate action we need to take only where it was an opening of the deal
-                deals_qty = readSQL("SELECT coalesce(SUM(ABS(qty)), 0) "   
-                                    "FROM deals AS d "
-                                    "LEFT JOIN sequence AS s ON s.id=d.close_sid "
-                                    "WHERE account_id=:account_id AND asset_id=:asset_id "
-                                    "AND (open_sid=:last_sid OR close_sid=:last_sid) AND s.type!=5",
-                                    [(":account_id", account_id), (":asset_id", asset_id), (":last_sid", last_sid)])
-                reminder = last_qty - deals_qty
-                # if last trade is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
-                if reminder > 0:
-                    last_sid -= 1
-            elif last_type == TransactionType.CorporateAction:
-                last_sid, ca_type = readSQL("SELECT d.close_sid, c.type FROM deals AS d "
-                                            "LEFT JOIN sequence AS s ON d.close_sid=s.id "
-                                            "LEFT JOIN corp_actions AS c ON s.operation_id=c.id "
-                                            "WHERE d.account_id=:account_id AND d.asset_id=:asset_id "
-                                            "ORDER BY d.close_sid DESC, d.open_sid DESC LIMIT 1",
-                                            [(":account_id", account_id), (":asset_id", asset_id)])
-                if ca_type == CorporateAction.Split \
-                        or ca_type == CorporateAction.StockDividend or ca_type == CorporateAction.SpinOff:
-                    last_sid -= 1
-                reminder = 0
-
-            # Get a list of all previous not matched trades or corporate actions of opposite direction (type parameter)
-            query = executeSQL("SELECT * FROM ("
-                               "SELECT s.id, ABS(t.qty), t.price FROM trades AS t "
-                               "LEFT JOIN sequence AS s ON s.type = 3 AND s.operation_id=t.id "
-                               "WHERE (:type)*qty < 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
-                               "AND s.id < :sid AND s.id > :last_sid "
-                               "UNION ALL "
-                               "SELECT s.id, "
-                               "CASE "
-                               "    WHEN c.type = 2 AND c.asset_id=:asset_id THEN c.qty "
-                               "    ELSE c.qty_new "
-                               "END AS qty, "
-                               "CASE "
-                               "    WHEN c.type = 2 AND c.asset_id=:asset_id THEN coalesce(l.value/c.qty, 0) "
-                               "    ELSE coalesce(l.value/c.qty_new, 0) "
-                               "END AS price "
-                               "FROM corp_actions AS c "
-                               "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
-                               "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=:asset_id AND l.value > 0 "
-                               "WHERE (:type)*c.qty_new < 0 AND (c.asset_id_new=:asset_id OR (c.asset_id=:asset_id AND c.type=2)) AND c.account_id=:account_id "
-                               "AND s.id < :sid AND s.id > :last_sid "
-                               ")ORDER BY id",
-                               [(":type", type), (":asset_id", asset_id), (":account_id", account_id),
-                                (":sid", seq_id), (":last_sid", last_sid)])
-            while query.next():  # Perform match ("closure") of previous trades
-                deal_sid, deal_qty, deal_price = readSQLrecord(query)    # deal_sid -> trade_sid
-                if reminder > 0:
-                    next_deal_qty = reminder
-                    reminder = 0
-                else:
-                    next_deal_qty = deal_qty
-                if (processed_qty + next_deal_qty) >= qty:  # We can't process more than qty of current trade
-                    next_deal_qty = qty - processed_qty     # If it happens - just process the remainder of the trade
-                # Create a deal with relevant sign of quantity (-1 for short, +1 for long)
-                _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_sid, close_sid, qty) "
-                               "VALUES(:account_id, :asset_id, :open_sid, :close_sid, :qty)",
-                               [(":account_id", account_id), (":asset_id", asset_id), (":open_sid", deal_sid),
-                                (":close_sid", seq_id), (":qty", (-type)*next_deal_qty)])
+            # Get a list of all previous not matched trades or corporate actions
+            query = executeSQL("SELECT timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty "
+                               "FROM open_trades "
+                               "WHERE account_id=:account_id AND asset_id=:asset_id AND remaining_qty!=0 "
+                               "ORDER BY timestamp, op_type DESC",
+                               [(":account_id", account_id), (":asset_id", asset_id)])
+            while query.next():
+                opening_trade = readSQLrecord(query, named=True)
+                next_deal_qty = opening_trade['remaining_qty']
+                if (processed_qty + next_deal_qty) > qty:  # We can't close all trades with current operation
+                    next_deal_qty = qty - processed_qty  # If it happens - just process the remainder of the trade
+                _ = executeSQL("UPDATE open_trades SET remaining_qty=remaining_qty-:qty "
+                               "WHERE op_type=:op_type AND operation_id=:id AND asset_id=:asset_id",
+                               [(":qty", next_deal_qty), (":op_type", opening_trade['op_type']),
+                                (":id", opening_trade['operation_id']), (":asset_id", asset_id)])
+                _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_op_type, open_op_id, open_timestamp, close_op_type, close_op_id, close_timestamp, qty) "
+                               "VALUES(:account_id, :asset_id, :open_op_type, :open_op_id, :open_timestamp, :close_op_type, :close_op_id, :close_timestamp, :qty)",
+                               [(":account_id", account_id), (":asset_id", asset_id), (":open_op_type", opening_trade['op_type']), (":open_op_id", opening_trade['operation_id']), (":open_timestamp", opening_trade['timestamp']),
+                                (":close_op_type", TransactionType.Trade), (":close_op_id", self.current['id']), (":close_timestamp", self.current['timestamp']), (":qty", (-type)*next_deal_qty)])
                 processed_qty += next_deal_qty
-                processed_value += (next_deal_qty * deal_price)
+                processed_value += (next_deal_qty * opening_trade['price'])
                 if processed_qty == qty:
                     break
         if type > 0:
@@ -356,6 +265,10 @@ class Ledger(QObject):
             self.current['category'] = PredefinedCategory.Profit
             self.appendTransaction(BookAccount.Incomes, type * ((price * processed_qty) - processed_value))
         if processed_qty < qty:  # We have reminder that opens a new position
+            _ = executeSQL("INSERT INTO open_trades(timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty) "
+                           "VALUES(:timestamp, :type, :operation_id, :account_id, :asset_id, :price, :remaining_qty)",
+                           [(":timestamp", self.current['timestamp']), (":type", TransactionType.Trade), (":operation_id", self.current['id']),
+                            (":account_id", account_id), (":asset_id", asset_id), (":price", price), (":remaining_qty", qty - processed_qty)])
             self.appendTransaction(BookAccount.Assets, type*(qty - processed_qty), type*(qty - processed_qty) * price)
         if self.current['fee_tax']:
             self.current['category'] = PredefinedCategory.Fees
@@ -401,7 +314,6 @@ class Ledger(QObject):
         if self.current['subtype'] == CorporateAction.StockDividend and self.current['amount'] < 0:
             self.updateStockDividendAssets()
 
-        seq_id = self.current_seq
         account_id = self.current['account']
         asset_id = self.current['asset']
         qty = self.current['amount']
@@ -414,69 +326,30 @@ class Ledger(QObject):
             logging.fatal(self.tr("Asset amount is not enough for corporate action processing. Date: ")
                           + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
             return
-        # Get information about last deal
-        last_sid = readSQL("SELECT "
-                           "CASE WHEN qty>0 THEN open_sid ELSE close_sid END AS last_sid "
-                           "FROM deals "
-                           "WHERE account_id=:account_id AND asset_id=:asset_id "
-                           "ORDER BY close_sid DESC, open_sid DESC LIMIT 1",
-                           [(":account_id", account_id), (":asset_id", asset_id)])
-        last_sid = 0 if last_sid is None else last_sid
-        # Next get information about abs trade quantity that was in this last deal
-        last_qty = readSQL("SELECT coalesce(ABS(t.qty), 0)+coalesce(ABS(ca.qty_new) , 0) AS qty "
-                           "FROM sequence AS s "
-                           "LEFT JOIN trades AS t ON t.id=s.operation_id AND s.type=3 "
-                           "LEFT JOIN corp_actions AS ca ON ca.id=s.operation_id AND s.type=5 "
-                           "WHERE s.id=:last_sid", [(":last_sid", last_sid)])
-        last_qty = 0 if last_qty is None else last_qty
-        # Collect quantity of all deals where this last opposite trade participated (positive value)
-        deals_qty = readSQL("SELECT coalesce(SUM(ABS(qty)), 0) "
-                            "FROM deals AS d "
-                            "WHERE account_id=:account_id AND asset_id=:asset_id "
-                            "AND (open_sid=:last_sid OR close_sid=:last_sid)",
-                            [(":account_id", account_id), (":asset_id", asset_id), (":last_sid", last_sid)])
-        reminder = last_qty - deals_qty
-        # if last trade is fully matched (reminder<=0) we start from next trade, otherwise we need to shift by 1
-        if reminder > 0:
-            last_sid = last_sid - 1
 
-        # Get a list of all previous not matched trades or corporate actions of opposite direction (type parameter)
-        query = executeSQL("SELECT * FROM ("
-                           "SELECT s.id, ABS(t.qty), t.price FROM trades AS t "
-                           "LEFT JOIN sequence AS s ON s.type = 3 AND s.operation_id=t.id "
-                           "WHERE qty > 0 AND t.asset_id = :asset_id AND t.account_id = :account_id "
-                           "AND s.id < :sid AND s.id > :last_sid "
-                           "UNION ALL "
-                           "SELECT s.id, ABS(c.qty_new), coalesce(l.value/c.qty_new, 0) AS price FROM corp_actions AS c "
-                           "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
-                           "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id_new AND l.value > 0 "
-                           "WHERE c.qty_new > 0 AND c.asset_id_new=:asset_id AND c.account_id=:account_id "
-                           "AND s.id < :sid AND s.id > :last_sid "
-                           "UNION ALL "  # Next part is for spin-offs that where data might be in old part
-                           "SELECT s.id, ABS(c.qty), coalesce(l.value/c.qty, 0) AS price FROM corp_actions AS c "
-                           "LEFT JOIN sequence AS s ON s.type = 5 AND s.operation_id=c.id "
-                           "LEFT JOIN ledger AS l ON s.id = l.sid AND l.asset_id=c.asset_id AND l.value > 0 "
-                           "WHERE c.qty_new > 0 AND c.asset_id=:asset_id AND c.type=:spinoff AND c.account_id=:account_id "
-                           "AND s.id < :sid AND s.id > :last_sid "
-                           ")ORDER BY id",
-                           [(":asset_id", asset_id), (":account_id", account_id), (":spinoff", CorporateAction.SpinOff),
-                            (":sid", seq_id), (":last_sid", last_sid)])
-        while query.next():  # Perform match ("closure") of previous trades
-            deal_sid, deal_qty, deal_price = readSQLrecord(query)  # deal_sid -> trade_sid
-            if reminder > 0:
-                next_deal_qty = reminder
-                reminder = 0
-            else:
-                next_deal_qty = deal_qty
-            if (processed_qty + next_deal_qty) >= qty:  # We can't process more than qty of current trade
-                next_deal_qty = qty - processed_qty  # If it happens - just process the remainder of the trade
+        # Get a list of all previous not matched trades or corporate actions
+        query = executeSQL("SELECT timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty "
+                           "FROM open_trades "
+                           "WHERE account_id=:account_id AND asset_id=:asset_id  AND remaining_qty!=0 "
+                           "ORDER BY timestamp, op_type DESC",
+                           [(":account_id", account_id), (":asset_id", asset_id)])
+        while query.next():
+            opening_trade = readSQLrecord(query, named=True)
+            next_deal_qty = opening_trade['remaining_qty']
+            if (processed_qty + next_deal_qty) > qty:  # We can't close all trades with current operation
+                raise ValueError(self.tr("Unhandled case: Corporate action covers not full open position"))
+            _ = executeSQL("UPDATE open_trades SET remaining_qty=0 "
+                           "WHERE op_type=:op_type AND operation_id=:id AND asset_id=:asset_id",
+                           [(":op_type", opening_trade['op_type']),(":id", opening_trade['operation_id']),
+                            (":asset_id", asset_id)])
+
             # Create a deal with relevant sign of quantity (-1 for short, +1 for long)
-            _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_sid, close_sid, qty) "
-                           "VALUES(:account_id, :asset_id, :open_sid, :close_sid, :qty)",
-                           [(":account_id", account_id), (":asset_id", asset_id), (":open_sid", deal_sid),
-                            (":close_sid", seq_id), (":qty", next_deal_qty)])
+            _ = executeSQL("INSERT INTO deals(account_id, asset_id, open_op_type, open_op_id, open_timestamp, close_op_type, close_op_id, close_timestamp, qty) "
+                           "VALUES(:account_id, :asset_id, :open_op_type, :open_op_id, :open_timestamp, :close_op_type, :close_op_id, :close_timestamp, :qty)",
+                           [(":account_id", account_id), (":asset_id", asset_id), (":open_op_type", opening_trade['op_type']),(":open_op_id", opening_trade['operation_id']), (":open_timestamp", opening_trade['timestamp']),
+                            (":close_op_type", TransactionType.CorporateAction), (":close_op_id", self.current['id']), (":close_timestamp", self.current['timestamp']), (":qty", next_deal_qty)])
             processed_qty += next_deal_qty
-            processed_value += (next_deal_qty * deal_price)
+            processed_value += (next_deal_qty * opening_trade['price'])
             if processed_qty == qty:
                 break
         # Asset allocations for different corporate actions:
@@ -497,10 +370,23 @@ class Ledger(QObject):
         new_value = processed_value
         if self.current['subtype'] == CorporateAction.SpinOff:
             new_value = processed_value * (1 - self.current['fee_tax'])
+            price = (processed_value - new_value) / self.current['amount']
             # Modify value for old asset
             self.appendTransaction(BookAccount.Assets, self.current['amount'], processed_value - new_value)
+            _ = executeSQL(
+                "INSERT INTO open_trades(timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty) "
+                "VALUES(:timestamp, :type, :operation_id, :account_id, :asset_id, :price, :remaining_qty)",
+                [(":timestamp", self.current['timestamp']), (":type", TransactionType.CorporateAction),
+                 (":operation_id", self.current['id']),
+                 (":account_id", account_id), (":asset_id", self.current['asset']), (":price", price), (":remaining_qty", self.current['amount'])])
         # Create value for new asset
         self.current['asset'] = new_asset
+        new_price = new_value / new_qty
+        _ = executeSQL("INSERT INTO open_trades(timestamp, op_type, operation_id, account_id, asset_id, price, remaining_qty) "
+                       "VALUES(:timestamp, :type, :operation_id, :account_id, :asset_id, :price, :remaining_qty)",
+                       [(":timestamp", self.current['timestamp']), (":type", TransactionType.CorporateAction),
+                        (":operation_id", self.current['id']),
+                        (":account_id", account_id), (":asset_id", new_asset), (":price", new_price), (":remaining_qty", new_qty)])
         self.appendTransaction(BookAccount.Assets, new_qty, new_value)
 
     # Rebuild transaction sequence and recalculate all amounts
@@ -518,10 +404,8 @@ class Ledger(QObject):
             TransactionType.CorporateAction: self.processCorporateAction
         }
 
-        # Initialize arrays for rolling sums storage
-        self.amounts = LedgerAmounts()
-        self.values = LedgerAmounts()
-        self.sids = {}
+        self.amounts.clear()
+        self.values.clear()
         if from_timestamp >= 0:
             frontier = from_timestamp
             operations_count = readSQL("SELECT COUNT(id) FROM all_transactions WHERE timestamp >= :frontier",
@@ -534,19 +418,21 @@ class Ledger(QObject):
                 if QMessageBox().warning(None, self.tr("Confirmation"), f"{operations_count}" +
                                          self.tr(" operations require rebuild. Do you want to do it right now?"),
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.No:
+                    JalSettings().setValue('RebuildDB', 1)
                     return
+        if operations_count == 0:
+            logging.info(self.tr("Leger is empty"))
+            return
         if self.progress_bar is not None:
             self.progress_bar.setRange(0, operations_count)
             self.main_window.showProgressBar(True)
         logging.info(self.tr("Re-building ledger since: ") +
                      f"{datetime.utcfromtimestamp(frontier).strftime('%d/%m/%Y %H:%M:%S')}")
         start_time = datetime.now()
-        _ = executeSQL("DELETE FROM deals WHERE close_sid >= "
-                       "(SELECT coalesce(MIN(id), 0) FROM sequence WHERE timestamp >= :frontier)",
-                       [(":frontier", frontier)])
+        _ = executeSQL("DELETE FROM deals WHERE close_timestamp >= :frontier", [(":frontier", frontier)])
         _ = executeSQL("DELETE FROM ledger WHERE timestamp >= :frontier", [(":frontier", frontier)])
-        _ = executeSQL("DELETE FROM sequence WHERE timestamp >= :frontier", [(":frontier", frontier)])
-        _ = executeSQL("DELETE FROM ledger_sums WHERE timestamp >= :frontier", [(":frontier", frontier)], commit=True)
+        _ = executeSQL("DELETE FROM ledger_totals WHERE timestamp >= :frontier", [(":frontier", frontier)])
+        _ = executeSQL("DELETE FROM open_trades WHERE timestamp >= :frontier", [(":frontier", frontier)])
 
         db_triggers_disable()
         if fast_and_dirty:  # For 30k operations difference of execution time is - with 0:02:41 / without 0:11:44
@@ -557,15 +443,6 @@ class Ledger(QObject):
                                "WHERE timestamp >= :frontier", [(":frontier", frontier)])
             while query.next():
                 self.current = readSQLrecord(query, named=True)
-                if self.current['type'] == TransactionType.Action:
-                    subtype = copysign(1, self.current['subtype'])
-                else:
-                    subtype = self.current['subtype']
-                seq_query = executeSQL("INSERT INTO sequence(timestamp, type, subtype, operation_id) "
-                                       "VALUES(:timestamp, :type, :subtype, :operation_id)",
-                                       [(":timestamp", self.current['timestamp']), (":type", self.current['type']),
-                                        (":subtype", subtype), (":operation_id", self.current['id'])])
-                self.current_seq = seq_query.lastInsertId()
                 operationProcess[self.current['type']]()
                 if self.progress_bar is not None:
                     self.progress_bar.setValue(query.at())
@@ -573,9 +450,18 @@ class Ledger(QObject):
             if fast_and_dirty:
                 _ = executeSQL("PRAGMA synchronous = ON")
             db_triggers_enable()
+        # Fill ledger totals values
+        _ = executeSQL("INSERT INTO ledger_totals"
+                       "(op_type, operation_id, timestamp, book_account, asset_id, account_id, amount_acc, value_acc) "
+                       "SELECT op_type, operation_id, timestamp, book_account, "
+                       "asset_id, account_id, amount_acc, value_acc FROM ledger "
+                       "WHERE id IN ("
+                       "SELECT MAX(id) FROM ledger WHERE timestamp >= :frontier "
+                       "GROUP BY op_type, operation_id, book_account, account_id)", [(":frontier", frontier)])
 
         if self.progress_bar is not None:
             self.main_window.showProgressBar(False)
+        JalSettings().setValue('RebuildDB', 0)
         logging.info(self.tr("Ledger is complete. Elapsed time: ") + f"{datetime.now() - start_time}" +
                      self.tr(", new frontier: ") + f"{datetime.utcfromtimestamp(self.current['timestamp']).strftime('%d/%m/%Y %H:%M:%S')}")
 
